# -*- coding: utf-8 -*-
#
#       adrift.py
#       
#       Copyright © 2009, 2010 Per Liedman <per@liedman.net>
#       Copyright © 2011 Brandon Invergo <b.invergo@gmail.com>
#       
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.


from binascii import hexlify
import md5

from storydata import StoryData
from ifformaterror import IFFormatError

class AdriftDecoder:
    INITIAL_STATE = 0x00A09E86
    SCRAMBLE1 = 0x43FD43FD
    SCRAMBLE2 = 0x00C39EC3
    SCRAMBLE3 = 0x00FFFFFF

    def __init__(self):
        self.state = AdriftDecoder.INITIAL_STATE

    def decode(self, character):
        self.state = (self.state * AdriftDecoder.SCRAMBLE1 +
                      AdriftDecoder.SCRAMBLE2) & AdriftDecoder.SCRAMBLE3
        r = 255 * self.state / (AdriftDecoder.SCRAMBLE3 + 1)
        return chr(r ^ ord(character))

    def skip(self, number_bytes):
        for x in range(0, number_bytes):
            self.decode('x')


def get_extensions():
    return ['taf']


def get_information(file_buffer):
    s = StoryData()
    if is_a(file_buffer):
        s.format = 'adrift'
    else:
        raise IFFormatError('file_buffer is not in adrift format.')
    s.ifid = calc_ifid(file_buffer)
    return s


def is_a(file_buffer):
    if len(file_buffer) < 12:
        return False
    else:
        decoder = AdriftDecoder()
        decoded_header = ''.join([decoder.decode(c) for c in file_buffer[0:7]])
        return decoded_header == 'Version'


def calc_ifid(file_buffer):
    decoder = AdriftDecoder()
    decoder.skip(8)
    ifid = 'ADRIFT-'
    ifid = ''.join([ifid, decoder.decode(file_buffer[8])])
    decoder.skip(1)
    ifid = ''.join([ifid, decoder.decode(file_buffer[10])])
    ifid = ''.join([ifid, decoder.decode(file_buffer[11])])
    ifid = ''.join([ifid, '-'])
    ifid = ''.join([ifid,  hexlify(md5.new(file_buffer).digest()).upper()])
    return ifid
