# -*- coding: utf-8 -*-
#
#       # -*- coding: utf-8 -*-
#
#       main.py
#       
#       Copyright © 2011 Brandon Invergo <b.invergo@gmail.com>
#       
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.

import struct

from storydata import StoryData
from _binaryfuncs import read_short, read_int
from ifformaterror import IFFormatError


AGX_MAGIC = (0x58, 0xC7, 0xC1, 0x51)


def get_extensions():
    return ['agx']

    
def get_information(file_buffer):
    s = StoryData()
    if is_a(file_buffer):
        s.format = 'agt'
    else:
        raise IFFormatError('file_buffer is not in agt format.')
    s.ifid = calc_ifid(file_buffer)
    return s


def calc_ifid(file_buffer):
    length = read_int(file_buffer, 32, '<')
    if len(file_buffer) < length + 6:
        return None
    game_version = read_short(file_buffer, length, '<')
    game_sig = read_int(file_buffer, length + 2, '<')
    return 'AGT-{0:05d}-{1:08X}'.format(game_version, game_sig)
    
    
def is_a(file_buffer):
    header = struct.unpack_from('<BBBB', file_buffer, 0)
    if (len(file_buffer) < 36 or header != AGX_MAGIC):
        return False
    return True


if __name__ == '__main__':
    import sys, struct
    data = open(sys.argv[1], 'rb').read()
    print is_a(data)
    print calc_ifid(data)
