# -*- coding: utf-8 -*-
#
#       main.py
#       
#       Copyright © 2011 Brandon Invergo <b.invergo@gmail.com>
#       
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.

import md5
from binascii import hexlify

from storydata import StoryData
from _binaryfuncs import read_long
from ifformaterror import IFFormatError


def get_extensions():
    return ['acd', 'a3c', 'a3r']
    
    
def get_information(file_buffer):
    s = StoryData()
    if is_a(file_buffer):
        s.format = 'alan'
    else:
        raise IFFormatError('file_buffer is not in alan format.')
    s.ifid = calc_ifid(file_buffer)
    return s
    

def calc_ifid(file_buffer):
    file_hash = hexlify(md5.new(file_buffer).digest()).upper()
    return ''.join(['ALAN-', file_hash])
    
    
def is_a(file_buffer):
    crc = 0
    if len(file_buffer) < 160:
        return False
    if not file_buffer.startswith('ALAN'):
        # Identify Alan 2.x
        bf = read_long(file_buffer, 4)
        if bf > len(file_buffer)/4:
            return False
        for i in range(24, 81, 4):
            if read_long(file_buffer, i) > len(file_buffer)/4:
                return False
        for i in range(160, bf * 4):
            crc += ord(file_buffer[i])
        if crc == read_long(file_buffer, 152):
            return True
    else:
        # Identify Alan 3.x
        crc = 0
        bf = read_long(file_buffer, 12)
        if bf > len(file_buffer)/4:
            return False
        for i in range(184, bf * 4):
            crc += ord(file_buffer[i])
        if crc == read_long(file_buffer, 176):
            return True
    return False


if __name__ == '__main__':
    import sys, struct
    data = open(sys.argv[1], 'rb').read()
    print is_a(data)
    print calc_ifid(data)
