# -*- coding: utf-8 -*-
#
#       babel.py
#       
#       Copyright © 2009, 2010 Per Liedman <per@liedman.net>
#       Copyright © 2011 Brandon Invergo <b.invergo@gmail.com>
#       
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.


import sys
import os.path

import adrift
import agt
import alan
import executable
import glulx
import hugo
import level9
import magscrolls
import tads
import zcode
import blorb
import ifiction
from ifformaterror import IFFormatError


class BabelError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


handlers = [adrift, agt, alan, executable, glulx, hugo, level9, magscrolls,
            tads, zcode]
extension_map = {}


for h in handlers:
    for ext in h.get_extensions():
        extension_map[ext] = h


def get_storydata(file_handle, filename, coverart_dir):
    file_buffer = file_handle.read()
    result = None
    coverart = None
    try:
        b = blorb.Blorb(file_buffer)
    except IFFormatError as e:
        extension = None
        if filename is not None:
            basename = os.path.basename(filename)
            extension = os.path.splitext(basename)[1].strip('.')
            if extension is not None and extension in extension_map:
                handler = extension_map[extension]
                if handler.is_a(file_buffer):
                    result = handler.get_information(file_buffer)
            if result is None:
                for handler in handlers:
                    if handler.is_a(file_buffer):
                        result = handler.get_information(file_buffer)
            if result is not None and result.format in ['tads2', 'tads3']:
                coverart = tads.get_cover(file_buffer)
    else:
        xml = b.get_metadata()
        if xml is not None:
            result = ifiction.get_stories_from_ifiction(xml)[0]
        else:
            story_format = b.get_story_format()
            if story_format is None:
                raise BabelError('Unknown story format.')
            story_file = b.get_story_file()
            handler = {"zcode": zcode, "glulx": glulx, "tads2": tads,
                       "tads3": tads, "adrift": adrift}.get(story_format)
            result = handler.get_information(story_file)
        coverart = b.get_cover()
    if result is not None:
        result.story_file = filename
    else:
        raise BabelError('Unknown story format.')
    if coverart is not None and coverart[1] is not None:
        coverart_file = os.path.join(coverart_dir,
                                     '.'.join([result.ifid, coverart[0]]))
        with open(coverart_file, 'wb') as coverart_handle:
            coverart_handle.write(coverart[1])
        result.coverart = coverart_file
    else:
        result.get_local_coverart(coverart_dir)
    return result


def get_storydata_from_filename(filename, coverart_dir):
    with open(filename, 'rb') as f:
        return get_storydata(f, filename, coverart_dir)


if __name__ == "__main__":
    name = sys.argv[1]
    with open(name, 'rb') as f:
        s = get_storydata(f, name, '')
    if s is not None:
         for attr in [attr for attr in dir(s) if not attr.startswith('_')]:
            print str(attr) + ": " + str(getattr(s, attr))
    else:
        print 'Not a valid story file ({0}).'.format(name)
    
