# -*- coding: utf-8 -*-
#
#       blorb.py
#       
#       Copyright © 2009, 2010 Per Liedman <per@liedman.net>
#       Copyright © 2011 Brandon Invergo <b.invergo@gmail.com>
#       
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.


from ifformaterror import IFFormatError
from _binaryfuncs import read_int


class Blorb:
    def __init__(self, data):
        self.data = data
        if len(self.data) < 16 or not self.data.startswith('FORM') \
                or not self.data[8:12] == 'IFRS':
            raise IFFormatError('Not a BLORB.')

    def get_chunk(self, chunk_id):
        i = 12
        while i < len(self.data) - 8:
            length = read_int(self.data, i + 4)
            if self.data[i:i + 4] == chunk_id:
                if length > self.data:
                    return (None, None)
                #return self.data[i + 8:i + 8 + length]
                return (i + 8, length)
            if length % 2 != 0:
                length = length + 1
            i = i + length + 8
        return (None, None)

    def get_resource(self, resource, n):
        i, length = self.get_chunk('RIdx')
        if i is None:
            return (None, None)
        ridx = self.data[i+4:]
        ridx_len = read_int(self.data, i)
        for i in range(ridx_len):
            if (ridx[i * 12:i * 12 + 4] == resource and
                    read_int(ridx, i * 12 + 4) == n):
                j = i
                i = read_int(ridx, j * 12 + 8)
                begin = i + 8
                out_len = read_int(self.data, i + 4)
                return (begin, out_len)
        return (None, None)

    def get_resources(self):
        index, length = self.get_chunk('RIdx')
        number_resources = read_int(index, 0)
        result = []
        for i in xrange(0, number_resources):
            rsc_name = index[4 + i * 12:8 + i * 12]
            rsc_number = read_int(index, 8 + i * 12)
            result.append((rsc_name, rsc_number))
        return result

    def get_metadata(self):
        index, length = self.get_chunk('IFmd')
        if index is None:
            return None
        return self.data[index:index + length]

    def get_story_format(self):
        treaty_registry = {'ZCOD': 'zcode', 'GLUL': 'glulx', 'TAD2': 'tads2',
                           'TAD3': 'tads3'}
        for story_format in treaty_registry:
            index, length = self.get_chunk(story_format)
            if index is not None:
                return treaty_registry[story_format]
        return None
    
    def get_story_file(self):
        index, length = self.get_resource('Exec', 0)
        if index is None:
            return None
        return self.data[index:index + length]
    
    def get_cover(self):
        i, length = self.get_chunk('Fspc')
        if length < 4:
            return (None, None)
        i = read_int(self.data, i)
        pict_i, pict_len = self.get_resource('Pict', i)
        if pict_i is None:
            return (None, None)
        cover = self.data[pict_i:pict_i + pict_len]
        cover_format = self.data[pict_i - 8:pict_i - 4]
        if cover_format == 'PNG ':
            return ('png', cover)
        elif cover_format == 'JPEG':
            return ('jpeg', cover)
        else:
            return (None, None)


if __name__ == "__main__":
    import sys
    name = sys.argv[1]
    with open(name) as f:
        data = f.read()
    b = Blorb(data)
    cover = b.get_cover()[1]
    print 'about to write'
    with open('temp.jpg', 'wb') as c:
        print 'writing'
        c.write(cover)
        
