# -*- coding: utf-8 -*-
#
#       babel.py
#       
#       Copyright © 2009, 2010 Per Liedman <per@liedman.net>
#       Copyright © 2011 Brandon Invergo <b.invergo@gmail.com>
#       
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.


import md5
from binascii import hexlify

from storydata import StoryData
import ifformaterror


elfmagic = ''.join([chr(x) for x in [0x7f, 0x45, 0x4c, 0x46]])
javamagic = ''.join([chr(x) for x in [0xCA, 0xFE, 0xBA, 0xBE]])
amigamagic = ''.join([chr(x) for x in [0, 0, 3, 0xe7]])
machomagic = ''.join([chr(x) for x in [0xFE, 0xED, 0xFA, 0xCE]])
                                 
exetypes = [("MZ", "MZ", 2),
           (elfmagic, "ELF", 4),
           (javamagic, "JAVA", 4),
           (amigamagic, "AMIGA", 4),
           ("#! ", "SCRIPT", 3),
           (machomagic, "MACHO",4),
           ("APPL", "MAC",4)] 


def get_extensions():
    return ['exe']


def get_information(file_buffer):
    s = StoryData()
    if is_a(file_buffer):
        s.format = 'executable'
    else:
        raise IFFormatError('file_buffer is not in executable format.')
    s.ifid = calc_ifid(file_buffer)
    return s
    
    
def calc_ifid(file_buffer):
    magic = deduce_magic(file_buffer)
    if magic is None:
        return None
    file_hash = hexlify(md5.new(file_buffer).digest()).upper()
    return ''.join([magic, '-', file_hash])
    
    
def is_a(file_buffer):
    if deduce_magic(file_buffer):
        return True
    return False
    

def deduce_magic(file_buffer):
    for magic, name, length in exetypes:
        if len(file_buffer) >= length and magic == file_buffer[0:length]:
            return name
    return None
