# -*- coding: utf-8 -*-
#
#       babel.py
#       
#       Copyright © 2009, 2010 Per Liedman <per@liedman.net>
#       Copyright © 2011 Brandon Invergo <b.invergo@gmail.com>
#       
#       This file is part of Grotesque.
#
#       Grotesque is free software: you can redistribute it and/or modify
#       it under the terms of the GNU General Public License as published by
#       the Free Software Foundation, either version 3 of the License, or
#       (at your option) any later version.
#
#       Grotesque is distributed in the hope that it will be useful,
#       but WITHOUT ANY WARRANTY; without even the implied warranty of
#       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#       GNU General Public License for more details.
#
#       You should have received a copy of the GNU General Public License
#       along with Grotesque.  If not, see <http://www.gnu.org/licenses/>.


import re
from binascii import hexlify

from storydata import StoryData
import ifformaterror
from _binaryfuncs import read_char


def get_extensions():
    return ['hex']


def get_information(file_buffer):
    s = StoryData()
    if is_a(file_buffer):
        s.format = 'hugo'
    else:
        raise IFFormatError('file_buffer is not in hugo format.')
    s.ifid = calc_ifid(file_buffer)
    return s
    

def calc_ifid(file_buffer):
    if len(file_buffer) < 11:
        return None
    ifid_match = re.search('UUID://([^/]+)//', file_buffer)
    if ifid_match is not None:
        return ifid_match.group(1)
    serial = file_buffer[3:11]
    serial = re.sub('\W', '-', serial)
    ifid = 'HUGO-{0:d}-{1:02X}-{2:02X}-{3}'.format(read_char(file_buffer, 0),
                                                 read_char(file_buffer, 1),
                                                 read_char(file_buffer, 2),
                                                 serial)
    return ifid


def is_a(file_buffer):
    if file_buffer is None or len(file_buffer) < 40:
        return False
    if ord(file_buffer[0]) < 34:
        scale = 4
    else:
        scale = 16
    for i in range(3, 11):
        if ord(file_buffer[i]) < 32 or ord(file_buffer[i]) > 126:
            return False
    for i in range(11, 24, 2):
        if read_hugo_addx(file_buffer, i) * scale > len(file_buffer):
            return False
    return True
 

def read_hugo_addx(file_buffer, i):
    return ord(file_buffer[i]) + ord(file_buffer[i + 1]) * 2**8
   
    
if __name__ == "__main__":
    import sys
    data = open(sys.argv[1]).read()
    print calc_ifid(data)
    print is_a(data)
